import java.io.*;
import java.net.*;

public class Monohilo extends Thread{

	private File fich;
	private Socket sock;
	private BufferedReader entrada;
	private PrintWriter salida;
	private BufferedReader fichIn;
	private PrintWriter fichOut;
	private static final int timeout=500;//500 milisegundos a la espera en metodos bloqueantes
	//Variables de control del hilo:
	private boolean life;
	private boolean cambios;
	private boolean cambiosOtrosHilos;
	private String stringCambios;
	private String stringOtrosHilos;

	public Monohilo(File fichero) throws IOException{
		fich=fichero;
		life=false;//Los hilos empiezan apagados
		cambios=false;//No tienen cambios
		cambiosOtrosHilos=false;
	}
	
	public void run(){
		try{
			while(true){
				if(life){
					//El hilo esta encendido
					//Leemos primera peticion (metodo bloqueante)
					//Definimos un tiempo infinito para realizar la conexion
					sock.setSoTimeout(0);
					String saludo=this.leerPeticion();
					System.out.println(saludo);
					//Comprobamos si la peticion es la correcta
					if(saludo.compareTo("hola servidor pizarra")!=0){
						System.out.println("Saludo incorrecto");
						//Cerramos los Sockets
						entrada.close();
						salida.close();
						sock.close();
						life=false;
						continue;
					}
					System.out.println("Saludo correcto (Iniciando echo)");
					//Enviamos contexto anterior
					this.mandarTodo();
					//Definimos un tiempo maximo para realizar la conexion
					sock.setSoTimeout(timeout);
					//Esperamos a que nos vaya enviando figuras
					String fig="";
					while(true){
						try{
							//Leemos figuras (metodo bloqueante)
							fig=this.leerPeticion();
							if(fig==(String) null){
								//Esto pasa cuando se corta el socket mientras espera
								break;
							}
							//Marcamos que se han producido cambios
							stringCambios=fig;
							cambios=true;
							System.out.println(fig);
							//Guardamos la figura
							this.guardarFigura(fig);
							//Realizamos un echo
							//this.mandarFigura(fig);
						}catch(SocketTimeoutException ste){
							if(cambiosOtrosHilos){
								//Hay cambios tenemos que actualizar
								this.mandarFigura(stringOtrosHilos);
								cambiosOtrosHilos=false;
							}
						}
					}
					System.out.println("Finalizada peticion");
					//Al acabar la peticion apagamos el hilo
					life=false;
					cambios=false;
					sock.close();
				}else{
					//El hilo esta apagado así que le mandamos a dormir
					//hasta que el maestro le active
					try{
						sleep(timeout);//Ponemos el hilo a dormir 0.5 segundos.
					}catch(InterruptedException ie){
						System.out.println("El hilo se ha despertado");
					}
				}
			}
		}catch(IOException ioe){
			System.out.println("El hilo ha terminado inesperadamente.\r\n"+ioe);
			life=false;
			cambios=false;
		}
	}
	
	public void setSocket(Socket conex) throws IOException{
		sock=conex;
		//Definimos un tiempo maximo para realizar la conexion
		//sock.setSoTimeout(timeout);
		//Preparamos los streams
		entrada=new BufferedReader(new InputStreamReader(sock.getInputStream()));
		salida=new PrintWriter(sock.getOutputStream());
	}
	
	public boolean stillAlive() throws IOException{
		return life;
	}
	
	public void go() throws IOException{
		life=true;
	}
	
	public boolean tieneCambios() throws IOException{
		return cambios;
	}
	
	
	public String leerCambios() throws IOException{
		cambios=false;
		return stringCambios;
	}
	
	public void mandarCambios(String str) throws IOException{
		cambiosOtrosHilos=true;
		stringOtrosHilos=str;
	}
	
	private String leerPeticion() throws IOException{
		return entrada.readLine();
	}
	
	private void mandarFigura(String figura) throws IOException{
		salida.println(figura);
		salida.flush();
	}
	
	private synchronized void mandarTodo() throws IOException{
		String leido="";
		fichIn=new BufferedReader(new FileReader(fich));
		while(true){
			leido=fichIn.readLine();
			if(leido==(String) null){
				break;
			}
			salida.println(leido);
			salida.flush();
		}
		fichIn.close();
		notifyAll();
	}
	
	private synchronized void guardarFigura(String figura) throws IOException{
		fichOut=new PrintWriter(new FileWriter(fich,true),true);//Append y autoflush
		fichOut.println(figura);
		notifyAll();//Para indicar a los otros hilos que pueden intentar acceder
	}
}
